//+------------------------------------------------------------------+
//|                                           Signal2MACrossDEMA.mqh |
//|                               Copyright (c) 2018-2024, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>

// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals of 2 MAs crosses                                   |
//| Type=SignalAdvanced                                              |
//| Name=2DEMA Cross                                                   |
//| ShortName=2MACross                                               |
//| Class=Signal2MACross                                             |
//| Page=signal_2mac                                                 |
//| Parameter=SlowPeriod,int,11,Slow MA period                       |
//| Parameter=FastPeriod,int,7,Fast Ma period                        |
//| Parameter=MAMethod,ENUM_MA_METHOD,MODE_LWMA,Method of averaging  |
//| Parameter=MAPrice,ENUM_APPLIED_PRICE,PRICE_OPEN,Price type       |
//| Parameter=Shift,int,0,Shift                                      |
//+------------------------------------------------------------------+
// wizard description end

class CiDEMAt : public CIndicator
{
protected:
   int               m_ma_period;
   int               m_ind_shift;
   int               m_applied;

public:
                     CiDEMAt(void);
                    ~CiDEMAt(void);
   // methods of access to protected data
   int               MaPeriod(void)        const { return(m_ma_period); }
   int               IndShift(void)        const { return(m_ind_shift); }
   int               Applied(void)         const { return(m_applied);   }
   // method of creation
   bool              Create(const string symbol, const ENUM_TIMEFRAMES period,
                            const int ma_period, const int ind_shift, const int applied);
   // methods of access to indicator data
   double            Main(const int index) const;
   // method of identifying
   virtual int       Type(void) const { return(IND_DEMA); }

protected:
   // methods of tuning
   virtual bool      Initialize(const string symbol, const ENUM_TIMEFRAMES period, const int num_params, const MqlParam &params[]);
   bool              Initialize(const string symbol, const ENUM_TIMEFRAMES period,
                                const int ma_period, const int ind_shift, const int applied);
};
  
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CiDEMAt::CiDEMAt(void) : m_ma_period(-1),
                       m_ind_shift(-1),
                       m_applied(-1)
{
}
  
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CiDEMAt::~CiDEMAt(void)
{
}

//+------------------------------------------------------------------+
//| Create indicator "Double Exponential Moving Average"             |
//+------------------------------------------------------------------+
bool CiDEMAt::Create(const string symbol, const ENUM_TIMEFRAMES period,
                    const int ma_period, const int ind_shift, const int applied)
{
   // check history
   if(!SetSymbolPeriod(symbol, period))
      return(false);
   ResetLastError();
   m_handle = iCustom(symbol, period, "Examples/DEMAtest", ma_period, ind_shift, applied);
   if(m_handle == INVALID_HANDLE)
   {
      Print("iCustom failed: ", _LastError);
      return(false);
   }
   // indicator successfully created
   if(!Initialize(symbol, period, ma_period, ind_shift, applied))
   {
      Print("Initialize failed: ", _LastError);
      // initialization failed
      IndicatorRelease(m_handle);
      m_handle = INVALID_HANDLE;
      return(false);
     }
     if(!ChartIndicatorAdd(0, 1, m_handle))
     {
      Print("ChartIndicatorAdd failed: ", _LastError);
   }

   return(true); // ok
}
  
//+------------------------------------------------------------------+
//| Initialize the indicator with universal parameters               |
//+------------------------------------------------------------------+
bool CiDEMAt::Initialize(const string symbol, const ENUM_TIMEFRAMES period, const int num_params, const MqlParam &params[])
{
   return(Initialize(symbol, period, (int)params[0].integer_value, (int)params[1].integer_value, (int)params[2].integer_value));
}

//+------------------------------------------------------------------+
//| Initialize indicator with the special parameters                 |
//+------------------------------------------------------------------+
bool CiDEMAt::Initialize(const string symbol, const ENUM_TIMEFRAMES period,
                        const int ma_period, const int ind_shift, const int applied)
{
   if(CreateBuffers(symbol, period, 1))
   {
      // string of status of drawing
      m_name = "DEMA";
      m_status = "(" + symbol + "," + PeriodDescription() + "," +
               IntegerToString(ma_period) + "," + IntegerToString(ind_shift) + "," +
               PriceDescription(applied) + ") H=" + IntegerToString(m_handle);
      //save settings
      m_ma_period = ma_period;
      m_ind_shift = ind_shift;
      m_applied = applied;
      // create buffers
      ((CIndicatorBuffer*)At(0)).Name("MAIN_LINE");
      ((CIndicatorBuffer*)At(0)).Offset(ind_shift);
      
      return(true); // ok
   }

   return(false); // error
}
  
//+------------------------------------------------------------------+
//| Access to buffer of "Double Exponential Moving Average"          |
//+------------------------------------------------------------------+
double CiDEMAt::Main(const int index) const
{
   CIndicatorBuffer *buffer=At(0);
   if(buffer==NULL)
      return(EMPTY_VALUE);
   return(buffer.At(index));
}

//+------------------------------------------------------------------+
//| Class Signal2MACross.                                            |
//| Purpose: Generator of trade signals based on 2 MAs crosses.      |
//+------------------------------------------------------------------+
class Signal2MACross : public CExpertSignal
{
  protected:
    CiDEMAt              m_maSlow;         // object-indicator
    CiDEMAt              m_maFast;         // object-indicator
    
    // adjustable parameters
    int               m_slow;
    int               m_fast;
    ENUM_MA_METHOD    m_method;
    ENUM_APPLIED_PRICE m_type;
    int               m_shift;
    
    // "weights" of market models (0-100)
    int               m_pattern_0;      // model 0 "fast MA crosses slow MA"

  public:
                      Signal2MACross(void);
                     ~Signal2MACross(void);
                     
    // parameters setters
    void              SlowPeriod(int value) { m_slow = value; }
    void              FastPeriod(int value) { m_fast = value; }
    void              MAMethod(ENUM_MA_METHOD value) { m_method = value; }
    void              MAPrice(ENUM_APPLIED_PRICE value) { m_type = value; }
    void              Shift(int value) { m_shift = value; }
    
    // adjusting "weights" of market models
    void              Pattern_0(int value) { m_pattern_0 = value; }
    
    // verification of settings
    virtual bool      ValidationSettings(void);
    
    // creating the indicator and timeseries
    virtual bool      InitIndicators(CIndicators *indicators);
    
    // checking if the market models are formed
    virtual int       LongCondition(void);
    virtual int       ShortCondition(void);

  protected:
    // initialization of the indicators
    bool              InitMAs(CIndicators *indicators);
    
    // helper functions to read indicators' data
    double            FastMA(int ind) { static double buffer[1]; m_maFast.GetData(ind, 1, 0, buffer); return buffer[0]; }
    double            SlowMA(int ind) { static double buffer[1]; m_maSlow.GetData(ind, 1, 0, buffer); return buffer[0]; }

    int StartIndex(void) override
    {
      const int base = m_type >= 10 ? iBars(_Symbol, PERIOD_CURRENT) - ::BarsCalculated(m_type) : 0;
      return((m_every_tick ? base : base + 1));
    }
};

//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
Signal2MACross::Signal2MACross(void) : m_slow(11), m_fast(7), m_method(MODE_LWMA), m_type(PRICE_OPEN), m_shift(0), m_pattern_0(100)
{
}

//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
Signal2MACross::~Signal2MACross(void)
{
}

//+------------------------------------------------------------------+
//| Validation settings protected data                               |
//+------------------------------------------------------------------+
bool Signal2MACross::ValidationSettings(void)
{
  if(!CExpertSignal::ValidationSettings()) return(false);
  return(true);
}

//+------------------------------------------------------------------+
//| Create indicators                                                |
//+------------------------------------------------------------------+
bool Signal2MACross::InitIndicators(CIndicators *indicators)
{
  if(indicators == NULL) return(false);
  if(!CExpertSignal::InitIndicators(indicators)) return(false);
  if(!InitMAs(indicators)) return(false);
  return(true);
}

//+------------------------------------------------------------------+
//| Create MA indicators                                             |
//+------------------------------------------------------------------+
bool Signal2MACross::InitMAs(CIndicators *indicators)
{
  if(indicators == NULL) return(false);

  // initialize object
  if(!m_maFast.Create(m_symbol.Name(), m_period, m_fast, m_shift, m_type)
  || !m_maSlow.Create(m_symbol.Name(), m_period, m_slow, m_shift, m_type))
  {
    printf(__FUNCTION__ + ": error initializing object");
    return(false);
  }
  
  m_maFast.Redrawer(true);
  m_maSlow.Redrawer(true);
  
  // add object to collection
  if(!indicators.Add(GetPointer(m_maFast))
  || !indicators.Add(GetPointer(m_maSlow)))
  {
    printf(__FUNCTION__ + ": error adding object");
    return(false);
  }
  
  return(true);
}

//+------------------------------------------------------------------+
//| "Voting" that price will grow                                    |
//+------------------------------------------------------------------+
int Signal2MACross::LongCondition(void)
{
  if(!TerminalInfoInteger(TERMINAL_TRADE_ALLOWED)) return 0;
  
  const int idx = StartIndex();
  
  if(FastMA(idx) > SlowMA(idx))
  {
    return m_pattern_0;
  }
  return 0;
}

//+------------------------------------------------------------------+
//| "Voting" that price will fall                                    |
//+------------------------------------------------------------------+
int Signal2MACross::ShortCondition(void)
{
  if(!TerminalInfoInteger(TERMINAL_TRADE_ALLOWED)) return 0;

  const int idx = StartIndex();

  if(FastMA(idx) < SlowMA(idx))
  {
    return m_pattern_0;
  }
  return 0;
}
//+------------------------------------------------------------------+
